@@ -32,6 +32,14 @@
 // and it has an associated dictionary which lists the protocol characteristics.
 #define	kIOPropertyProtocolCharacteristicsKey		"Protocol Characteristics"
 
+// An identifier that will uniquely identify this SCSI Domain for the Physical Interconnect type.
+// This identifier is only guaranteed to be unique for any given Physical Interconnect and is
+// not guaranteed to be the same across restarts or shutdowns.
+#define kIOPropertySCSIDomainIdentifierKey			"SCSI Domain Identifier"
+
+// This is the SCSI Target Identifier for a given SCSI Target Device
+#define kIOPropertySCSITargetIdentifierKey			"SCSI Target Identifier"
+
 // This key is used to define the Physical Interconnect to which a device is
 // attached (e.g. ATAPI, FireWire).
 #define kIOPropertyPhysicalInterconnectTypeKey		"Physical Interconnect"
